#lang racket
(require "survey-gui.rkt"
         "ops.rkt"
         (for-syntax syntax/parse))


(provide form
         (rename-out [when* when])
         
         (rename-out [boolean-widget boolean]
                     [money-widget money])
         
         #%app
         #%datum
         
         #%module-begin)

(provide-ops)

;; ----------------------------------------

(define-syntax form
  (lambda (stx)
    (syntax-parse stx
      [(_ name clause ...)
       (unless (identifier? #'name)
         (raise-syntax-error 'form
                             "expected an identifier for the form"
                             #'name))
       #'(begin
           (define name (make-gui 'name))
           (form-clause name (lambda () #t) clause) ...
           (send name start))])))

(define-syntax form-clause
  (lambda (stx)
    (syntax-parse stx #:literals (when*)
      [(_ form-name guard-proc (when* expr nested-clause ...))
       #'(begin
           (define new-guard-proc (lambda () (and (guard-proc) (?/coerce expr))))
           (form-clause form-name new-guard-proc nested-clause)
           ...)]
      [(_ form-name guard-proc [id question type])
       #'(form-clause* form-name guard-proc id question type #f)]
      [(_ form-name guard-proc [id question type expr])
       #'(form-clause* form-name guard-proc id question type (lambda () expr))]
      [(_ form-name guard-proc other)
       (raise-syntax-error 'form
                           "bad question format"
                           #'other)])))

(define-syntax form-clause*
  (lambda (stx)
    (syntax-parse stx
      [(_ form-name guard-proc id question type get-value)
       (unless (identifier? #'id)
         (raise-syntax-error 'form
                             "expected an identifier for the question; found something else"
                             #'id))
       (unless (string? (syntax-e #'question))
         (raise-syntax-error 'form
                             "expected an string for the question; found something else"
                             #'question))
       #'(begin
           (define id undefined)
           (gui-add! form-name
                     type
                     question
                     guard-proc
                     (lambda (v) (set! id v))
                     get-value))])))

(define-syntax when*
  (lambda (stx)
    (raise-syntax-error #f
                        "misuse of keyword"
                        stx)))

