#lang racket
(require "survey-gui.rkt"
         "ops.rkt"
         (for-syntax syntax/parse))


(provide form
         (rename-out [when* when])
         
         (rename-out [boolean-widget boolean]
                     [money-widget money])
         
         (rename-out [-/typed -]
                     [>/typed >]
                     [</typed <]
                     [=/typed =]
                     [and/typed and]
                     [or/typed or])
         
         #%app
         (rename-out [datum #%datum])
         #%module-begin)

;; ----------------------------------------

(define-syntax form
  (lambda (stx)
    (syntax-parse stx
      [(_ name clause ...)
       (unless (identifier? #'name)
         (raise-syntax-error 'form
                             "expected an identifier for the form"
                             #'name))
       #'(begin
           (define name (make-gui 'name))
           (form-clause name (lambda () #t) clause) ...
           (send name start))])))

(define-syntax form-clause
  (lambda (stx)
    (syntax-parse stx #:literals (when*)
      [(_ form-name guard-proc (when* expr nested-clause ...))
       #'(begin
           (define new-guard-proc (lambda () 
                                    (and (guard-proc) 
                                         (?/coerce (check-type expr boolean)))))
           (form-clause form-name new-guard-proc nested-clause)
           ...)]
      [(_ form-name guard-proc [id question type])
       #'(form-clause* form-name guard-proc id question type #f)]
      [(_ form-name guard-proc [id question type expr])
       #'(form-clause* form-name guard-proc id question type 
                       (lambda () (check-type expr type)))]
      [(_ form-name guard-proc other)
       (raise-syntax-error 'form
                           "bad question format"
                           #'other)])))
    
(define-syntax form-clause*
  (lambda (stx)
    (syntax-parse stx
      [(_ form-name guard-proc id question type get-value)
       (unless (identifier? #'id)
         (raise-syntax-error 'form
                             "expected an identifier for the question; found something else"
                             #'id))
       (unless (string? (syntax-e #'question))
         (raise-syntax-error 'form
                             "expected an string for the question; found something else"
                             #'question))
       #'(begin
           (define val-id undefined)
           (define-syntax id (typed #'val-id #'type))
           (gui-add! form-name
                     type
                     question
                     guard-proc
                     (lambda (v) (set! val-id v))
                     get-value))])))

(define-syntax when*
  (lambda (stx)
    (raise-syntax-error #f
                        "misuse of keyword"
                        stx)))

;; ----------------------------------------

(begin-for-syntax
  (define (typed id type)
    (lambda (stx)
      (if (identifier? stx)
          #`(has-type #,id #,type)
          (raise-syntax-error
           #f
           "cannot use variable as a function"
           stx)))))

(define-syntax-rule (has-type expr type)
  expr)

(define-syntax check-type
  (lambda (stx)
    (syntax-parse stx
      [(_ expr expected-type)
       (let ([expr2 (local-expand #'expr
                                  'expression
                                  (list #'has-type))])
         (syntax-parse expr2 #:literals (has-type)
           [(has-type v type)
            (unless (equal? (syntax-e #'type)
                            (syntax-e #'expected-type))
              (raise-syntax-error
               'form
               (format "expected type ~a, found type ~a"
                       (syntax-e #'expected-type)
                       (syntax-e #'type))
               #'expr))
            expr2]
           [_
            (raise-syntax-error
             'form
             (format "expected type ~a, found expression of unknown type"
                     (syntax-e #'expected-type))
             #'expr)]))])))

(define-syntax datum
  (lambda (stx)
    (syntax-parse stx
      [(_ . val)
       (define raw-val (syntax-e #'val))
       (define type-id
         (cond
           [(number? raw-val) #'money]
           [(boolean? raw-val) #'boolean]
           [else #f]))
       (if type-id
           #`(has-type (#%datum . val) #,type-id)
           #`(#%datum . val))])))

(define-syntax-rule (define-binary-typed op orig-op type1 type2 rtype)
  (define-syntax-rule (op expr1 expr2)
    (has-type (orig-op (check-type expr1 type1)
                       (check-type expr2 type2))
              rtype)))

(define-binary-typed -/typed -/coerce money money money)
(define-binary-typed </typed </coerce money money boolean)
(define-binary-typed >/typed >/coerce money money boolean)
(define-binary-typed =/typed =/coerce money money boolean)
(define-binary-typed and/typed and/coerce boolean boolean boolean)
(define-binary-typed or/typed or/coerce boolean boolean boolean)
